package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.DriverMetainfo
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class EntityNotImplementedException : SchemaDefinitionException, CopyableThrowable<EntityNotImplementedException> {
	public val provider: Any
	public val driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>

	public constructor(provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>) : super(formatMessage(provider, driver)) {
		this.provider = provider
		this.driver = driver
	}

	public constructor(provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>, customMessage: String) : super(customMessage) {
		this.provider = provider
		this.driver = driver
	}

	public constructor(provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.provider = provider
		this.driver = driver
	}

	public constructor(provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>, cause: Throwable) : super(formatMessage(provider, driver), cause) {
		this.provider = provider
		this.driver = driver
	}

	public constructor(
		provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.provider = provider
		this.driver = driver
	}

	override fun createCopy(): EntityNotImplementedException? {
		return EntityNotImplementedException(
			provider = this.provider,
			driver = this.driver,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			provider: Any, driver: DriverMetainfo<*, *, *, *, *, *, *, *, *, *>
		): String = "$provider doesn't provides implementation for driver $driver"
	}
}