package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException
import ru.landgrafhomyak.db.skeleton1.api.module.Module

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class UnknownVersionException : SchemaDefinitionException, CopyableThrowable<UnknownVersionException> {
	public val template: Module.Template<*>
	public val version: String

	public constructor(template: Module.Template<*>, version: String) : super(formatMessage(template, version)) {
		this.template = template
		this.version = version
	}

	public constructor(template: Module.Template<*>, version: String, customMessage: String) : super(customMessage) {
		this.template = template
		this.version = version
	}

	public constructor(template: Module.Template<*>, version: String, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.template = template
		this.version = version
	}

	public constructor(template: Module.Template<*>, version: String, cause: Throwable) : super(formatMessage(template, version), cause) {
		this.template = template
		this.version = version
	}

	public constructor(
		template: Module.Template<*>, version: String,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.template = template
		this.version = version
	}

	override fun createCopy(): UnknownVersionException? {
		return UnknownVersionException(
			template = this.template,
			version = this.version,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			template: Module.Template<*>, version: String
		): String = "Template $template doesn't knows version '${version}'"
	}
}