package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException
import ru.landgrafhomyak.db.skeleton1.api.module.Module

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class VersionDuplicationException : SchemaDefinitionException, CopyableThrowable<VersionDuplicationException> {
	public val template: Module.Template<*>
	public val versionOwner: Module<*, *>
	public val newModule: Module<*, *>
	public val version: String

	public constructor(template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String) : super(formatMessage(template, versionOwner, newModule, version)) {
		this.template = template
		this.versionOwner = versionOwner
		this.newModule = newModule
		this.version = version
	}

	public constructor(template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String, customMessage: String) : super(customMessage) {
		this.template = template
		this.versionOwner = versionOwner
		this.newModule = newModule
		this.version = version
	}

	public constructor(template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.template = template
		this.versionOwner = versionOwner
		this.newModule = newModule
		this.version = version
	}

	public constructor(template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String, cause: Throwable) : super(formatMessage(template, versionOwner, newModule, version), cause) {
		this.template = template
		this.versionOwner = versionOwner
		this.newModule = newModule
		this.version = version
	}

	public constructor(
		template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.template = template
		this.versionOwner = versionOwner
		this.newModule = newModule
		this.version = version
	}

	override fun createCopy(): VersionDuplicationException? {
		return VersionDuplicationException(
			template = this.template,
			versionOwner = this.versionOwner,
			newModule = this.newModule,
			version = this.version,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			template: Module.Template<*>, versionOwner: Module<*, *>, newModule: Module<*, *>, version: String
		): String = "Version '${version}' already associated with $versionOwner and $newModule provided by $template has the same version (circular upgrades are prohibited)"
	}
}