package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException
import ru.landgrafhomyak.db.skeleton1.api.module.Module

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class WrongScopeException : SchemaDefinitionException, CopyableThrowable<WrongScopeException> {
	public val wrongObject: Any
	public val currentScope: Any
	public val correctScope: Any

	public constructor(wrongObject: Any, currentScope: Any, correctScope: Any) : super(formatMessage(wrongObject, currentScope, correctScope)) {
		this.wrongObject = wrongObject
		this.currentScope = currentScope
		this.correctScope = correctScope
	}

	public constructor(wrongObject: Any, currentScope: Any, correctScope: Any, customMessage: String) : super(customMessage) {
		this.wrongObject = wrongObject
		this.currentScope = currentScope
		this.correctScope = correctScope
	}

	public constructor(wrongObject: Any, currentScope: Any, correctScope: Any, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.wrongObject = wrongObject
		this.currentScope = currentScope
		this.correctScope = correctScope
	}

	public constructor(wrongObject: Any, currentScope: Any, correctScope: Any, cause: Throwable) : super(formatMessage(wrongObject, currentScope, correctScope), cause) {
		this.wrongObject = wrongObject
		this.currentScope = currentScope
		this.correctScope = correctScope
	}

	public constructor(
		wrongObject: Any, currentScope: Any, correctScope: Any,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.wrongObject = wrongObject
		this.currentScope = currentScope
		this.correctScope = correctScope
	}

	override fun createCopy(): WrongScopeException? {
		return WrongScopeException(
			wrongObject = this.wrongObject,
			currentScope = this.currentScope,
			correctScope = this.correctScope,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			wrongObject: Any, currentScope: Any, correctScope: Any
		): String = "Entity $wrongObject passed to wrong scope $currentScope (must be $correctScope)"
	}
}