package ru.landgrafhomyak.db.skeleton1.api.module

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.QueriesCompiler
import ru.landgrafhomyak.db.skeleton1.api.runtime.Transaction


public interface Module<mUE : Any, IK : Any> {
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("uExt")
	public val uExt: mUE

	@DebugApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getVersionKey")
	public val versionKey: String

	@DebugApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getParent")
	public val parent: Module<*, IK>?

	public interface Constructor<mUE : Any, IK : Any> {
		@Suppress("INAPPLICABLE_JVM_NAME")
		@get:JvmName("versionKey")
		public val versionKey: String

		public fun createSchema(context: ModuleConstructorScope<mUE, IK>): mUE

		public suspend fun initData(ext: mUE, compiler: QueriesCompiler<IK>, transaction: Transaction<IK>) {}
	}

	public interface Upgrade<nmUE : Any, omUE : Any, IK : Any> {
		@Suppress("INAPPLICABLE_JVM_NAME")
		@get:JvmName("versionKey")
		public val versionKey: String

		public fun upgradeSchema(context: ModuleUpgradeScope<nmUE, omUE, IK>): nmUE

		public suspend fun upgradeData(ext: nmUE, compiler: QueriesCompiler<IK>, transaction: Transaction<IK>) {}
	}

	public interface Template<@Suppress("unused") mUE : Any> {
		@LowLevelApi
		public fun <IK : Any> getConstructor(): Constructor<*, IK>

		@LowLevelApi
		public fun <IK : Any> getUpgrade(version: String): Upgrade<*, *, IK>?
	}
}