package ru.landgrafhomyak.db.skeleton1.api.module

import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.RawStatement
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement
import ru.landgrafhomyak.db.skeleton1.api.runtime.InputRow
import ru.landgrafhomyak.db.skeleton1.api.table.Table


public interface ModuleUpgradeScope<nmUE : Any, omUE : Any, IK : Any> : ModuleConstructorScope<nmUE, IK> {
	// Table -------------------------------------------------------------------
	public fun <tUE : Any> keepTable(table: Table<tUE, omUE, IK>): Table<tUE, nmUE, IK>
	public fun <tUE : Any> keepAndRenameTable(
		table: Table<tUE, omUE, IK>,
		newNamespace: Namespace<nmUE, IK>? = null, newName: String
	): Table<tUE, nmUE, IK>

	public fun <ntUE : Any, otUE : Any> upgradeTable(
		oldTable: Table<otUE, omUE, IK>,
		upgrade: Table.Upgrade<ntUE, otUE, IK>
	): Table<ntUE, nmUE, IK>

	public fun <ntUE : Any, otUE : Any> upgradeAndRenameTable(
		table: Table<otUE, omUE, IK>,
		newNamespace: Namespace<nmUE, IK>? = null, newName: String,
		upgrade: Table.Upgrade<ntUE, otUE, IK>
	): Table<ntUE, nmUE, IK>

	public fun deleteTable(table: Table<*, omUE, IK>)
	public fun <tUE : Any> deleteTableAfterModuleUpgrade(table: Table<tUE, omUE, IK>): Table<tUE, nmUE, IK>

	public fun exportTable(table: Table<*, omUE, IK>)
	public fun <tUE : Any> importTable(
		table: Table<tUE, *, IK>,
		namespace: Namespace<nmUE, IK> = this.rootNs, name: String
	): Table<tUE, nmUE, IK>

	public fun <ntUE : Any, otUE : Any> importAndUpgradeTable(
		table: Table<otUE, *, IK>,
		namespace: Namespace<nmUE, IK> = this.rootNs, name: String,
		upgrade: Table.Upgrade<ntUE, otUE, IK>
	): Table<ntUE, nmUE, IK>

	// Object ------------------------------------------------------------------

	public fun <oUE : Any?> keepModuleScopedObject(obj: ModuleScopedObject<oUE, omUE, IK>): ModuleScopedObject<oUE, nmUE, IK>
	public fun <oUE : Any?> renameModuleScopedObject(
		obj: ModuleScopedObject<oUE, omUE, IK>,
		newNamespace: Namespace<nmUE, IK>? = null, newName: String
	): ModuleScopedObject<oUE, nmUE, IK>

	public fun <oUE : Any?> deleteModuleScopedObject(obj: ModuleScopedObject<oUE, omUE, IK>)
	public fun <oUE : Any?> deleteModuleScopedObjectAfterModuleUpgrade(obj: ModuleScopedObject<oUE, omUE, IK>): ModuleScopedObject<oUE, nmUE, IK>

	public fun exportModuleScopedObject(obj: ModuleScopedObject<*, omUE, IK>)
	public fun <oUE : Any> importModuleScopedObject(
		obj: ModuleScopedObject<oUE, *, IK>,
		namespace: Namespace<nmUE, IK> = this.rootNs, name: String
	): ModuleScopedObject<oUE, nmUE, IK>


	// Module ------------------------------------------------------------------

	public fun <smUE : Any> keepModule(module: Module<smUE, IK>): Module<smUE, IK>
	public fun <smUE : Any> keepAndMoveModule(oldModule: Module<smUE, IK>, newNs: Namespace<nmUE, IK>): Module<smUE, IK>

	public fun <smUE : Any> upgradeModule(oldModule: Module<*, IK>, template: Module.Template<smUE>): Module<smUE, IK>
	public fun <smUE : Any> upgradeAndMoveModule(
		oldModule: Module<*, IK>,
		newNamespace: Namespace<nmUE, IK>,
		template: Module.Template<smUE>
	): Module<smUE, IK>

	public fun deleteModule(oldModule: Module<*, IK>)
	public fun <smUE : Any> deleteModuleAfterUpgrade(oldModule: Module<smUE, IK>): Module<smUE, IK>

	public fun exportModule(table: Module<*, IK>)

	public fun <smUE : Any> importModule(
		module: Module<smUE, IK>,
		namespace: Namespace<nmUE, IK>,
	): Module<smUE, IK>

	public fun <nsmUE : Any, osmUE : Any> importAndUpgradeModule(
		module: Module<osmUE, IK>,
		namespace: Namespace<nmUE, IK>,
		upgrade: Module.Upgrade<nsmUE, osmUE, IK>
	): Module<nsmUE, IK>

	// Statements --------------------------------------------------------------

	public fun addStatementAfterUpgrade(stmt: _Statement.Void2Void<*, IK>)
	public fun addStatementAfterUpgrade(stmt: RawStatement.Void2Void.ImplementationsProvider<*>)

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: _Statement.Params2Void<sUE, IK>): InputRow._Scope<sUE, IK, Unit>

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: RawStatement.Params2Void.ImplementationsProvider<sUE>): InputRow._Scope<sUE, IK, Unit>
}